@@ -1,4 +1,5 @@
 import copy
+import hashlib
 import logging
 import os
 from packaging import version
@@ -236,21 +237,10 @@ class Discourse(object):
     @staticmethod
     def enforce_group_name_length(name):
         if len(name) > Discourse.GROUP_NAME_LENGTH_MAX:
-            shorter = 'g' + name[-(Discourse.GROUP_NAME_LENGTH_MAX - 1):]
-            logger.error(
-                f'group name {name} length > {Discourse.GROUP_NAME_LENGTH_MAX}'
-                f' truncated to {shorter}')
-            name = shorter
+            md5 = hashlib.md5(name.encode('utf-8')).hexdigest()
+            name = 'g' + md5[:Discourse.GROUP_NAME_LENGTH_MAX - 1]
         return name
 
-    @staticmethod
-    def category2moderators_group(name):
-        return Discourse.enforce_group_name_length(f'{name}-mo')
-
-    @staticmethod
-    def category2members_group(name):
-        return Discourse.enforce_group_name_length(f'{name}-me')
-
     @staticmethod
     def category_fields():
         return (
@@ -426,6 +416,7 @@ class Discourse(object):
     @staticmethod
     def group_fields():
         return ('name',
+                'full_name',
                 'id',
                 'public_exit',
                 'members_visibility_level',
