@@ -49,49 +49,64 @@ class Importer(object):
     def group_members_name(self):
         return Discourse.enforce_group_name_length(self.category2members_group(self.name_get()))
 
+    def user_create(self, email, name):
+        username = None
+        if name:
+            candidate = re.sub(r'^[\W_]*(.*?)[\W_]*$', r'\1', unidecode.unidecode(name))
+            candidate = re.sub(r'[\W_]+', r'_', candidate)
+            too_short = len(candidate) < Discourse.USERNAME_MIN_LENGTH
+            too_long = len(candidate) > Discourse.USERNAME_MAX_LENGTH
+            if (not too_short and not too_long and not self.discourse.user_username_exists(
+                    candidate)):
+                username = candidate
+        if not username:
+            username = hashlib.md5(email.encode('utf-8')).hexdigest()[:20]
+        password = hashlib.md5(os.urandom(16)).hexdigest()[:20]
+        return self.discourse.user_create(email, **{
+            'username': username,
+            'password': password,
+            'active': 'true',
+            'name': name})
+
     def users_add(self):
         before = {}
         after = {}
         for email in [e for e in self.mailman.user.keys()]:
+            if self.args.preserve_existing_user_preferences:
+                existing = self.discourse.user_get(email)
+                if existing is not None and existing['active'] == 'true':
+                    before[email], after[email] = existing, existing
+                    continue
             user = self.mailman.user[email]
             name = user.get('usernames', '')
-            username = None
-            if name:
-                candidate = re.sub(r'^[\W_]*(.*?)[\W_]*$', r'\1', unidecode.unidecode(name))
-                candidate = re.sub(r'[\W_]+', r'_', candidate)
-                too_short = len(candidate) < Discourse.USERNAME_MIN_LENGTH
-                too_long = len(candidate) > Discourse.USERNAME_MAX_LENGTH
-                if (not too_short and not too_long and not self.discourse.user_username_exists(
-                        candidate)):
-                    username = candidate
-            if not username:
-                username = hashlib.md5(email.encode('utf-8')).hexdigest()[:20]
-            password = hashlib.md5(os.urandom(16)).hexdigest()[:20]
             try:
-                before[email], after[email] = self.discourse.user_create(email, **{
-                    'username': username,
-                    'password': password,
-                    'active': 'true',
-                    'name': name})
+                before[email], after[email] = self.user_create(email, name)
             except Exception as e:
                 logger.error(f'SKIP user {user} because {e}')
                 del self.mailman.user[email]
         return before, after
 
-    def users_preferences(self):
+    def users_preferences(self, existing_users):
         before = {}
         after = {}
         default_preferences = {}
         if self.mailman.info['private_roster'] != Mailman.ROSTER_PUBLIC:
             default_preferences['hide_profile_and_presence'] = 'true'
         for email, user in self.mailman.user.items():
-            preferences = copy.deepcopy(default_preferences)
-            language = user.get('language')
-            if language:
-                preferences['locale'] = language
-            if preferences != {}:
-                before[email], after[email] = self.discourse.user_preferences_set(
-                    email, **preferences)
+            if (
+                    self.args.preserve_existing_user_preferences and
+                    existing_users[email] is not None
+            ):
+                existing_preferences = self.discourse.user_preferences_get(email)
+                before[email], after[email] = existing_preferences, existing_preferences
+            else:
+                preferences = copy.deepcopy(default_preferences)
+                language = user.get('language')
+                if language:
+                    preferences['locale'] = language
+                if preferences != {}:
+                    before[email], after[email] = self.discourse.user_preferences_set(
+                        email, **preferences)
         return before, after
 
     def users_notifications(self):
@@ -306,7 +321,7 @@ class Importer(object):
         (_, after['category']) = self.discourse.category_set(
             self.name_get(), **self.category_settings())
 
-        (before['preferences'], after['preferences']) = self.users_preferences()
+        (before['preferences'], after['preferences']) = self.users_preferences(before['members'])
         (before['notifications'], after['notifications']) = self.users_notifications()
 
         return before, after
