@@ -25,4 +25,5 @@ def test_options(version, tmpdir):
                               api_key=api_key,
                               api_user='api',
                               url=url,
-                              mailman_encoding='UTF-8')
+                              mailman_encoding='UTF-8',
+                              preserve_existing_user_preferences=False)
