@@ -85,6 +85,86 @@ def test_users_add_normalize(test_options, raw, cooked):
         i.discourse.user_delete(email)
 
 
+def test_users_add_preserve_existing_user_preference(test_options):
+    i = importer.Importer(test_options)
+    i.args.preserve_existing_user_preferences = True
+    email = 'user@example.com'
+    goodname = 'goodname'
+    try:
+        #
+        # First time around, the user is created and the name is set
+        #
+        i.mailman.user = {
+            email: {
+                'usernames': goodname,
+            },
+        }
+        before, after = i.users_add()
+        assert before[email] is None
+        assert after[email]['name'] == goodname
+
+        #
+        # Second time around, the user exists and the name is not overridden
+        #
+        i.mailman.user = {
+            email: {
+                'usernames': 'ignoredname',
+            },
+        }
+        before, after = i.users_add()
+        assert before[email]['name'] == goodname
+        assert after[email]['name'] == goodname
+
+    finally:
+        i.discourse.user_delete(email)
+
+
+def test_users_add_deactivated_preserve_existing_user_preference(test_options):
+    i = importer.Importer(test_options)
+    i.args.preserve_existing_user_preferences = True
+    email = 'user@example.com'
+    goodname = 'goodname'
+    try:
+        #
+        # First time around, the user is created and the name is set
+        #
+        i.mailman.user = {
+            email: {
+                'usernames': goodname,
+            },
+        }
+        before, after = i.users_add()
+        assert before[email] is None
+        assert after[email]['name'] == goodname
+
+        #
+        # The user is deactivated
+        #
+        assert after[email]['active'] == 'true'
+        user_before, user_after = i.discourse.user_create(email, active='false')
+        assert user_before['active'] == 'true'
+        assert user_after['active'] == 'false'
+
+        #
+        # Second time around, the user exists but it is updated anyways
+        # because it needs to be activated, and the name is overriden
+        #
+        newname = 'newname'
+        i.mailman.user = {
+            email: {
+                'usernames': newname,
+            },
+        }
+        before, after = i.users_add()
+        assert before[email]['active'] == 'false'
+        assert before[email]['name'] == goodname
+        assert after[email]['active'] == 'true'
+        assert after[email]['name'] == newname
+
+    finally:
+        i.discourse.user_delete(email)
+
+
 def test_importer_moderation_no(test_options):
     i = importer.Importer(test_options)
     owner = 'owner@example.com'
@@ -444,6 +524,38 @@ def test_importer_user_preferences(test_options, private_roster):
         i.discourse.user_delete(no_language)
 
 
+def test_importer_user_preferences_preserve_existing_user_preferences(test_options):
+    i = importer.Importer(test_options)
+    i.args.preserve_existing_user_preferences = True
+    email = 'user@example.com'
+    try:
+        c = i.mailman.load_pickle()
+        mailman_add(c, email, language='fr')
+        mailman_write_config(test_options.mailman_config, c)
+        i.mailman.load()
+
+        #
+        # language is set to 'fr' when the preferences are set for the first time
+        #
+        before0, after0 = i.importer()
+        assert before0['members'][email] is None
+        assert after0['preferences'][email]['locale'] == 'fr'
+
+        #
+        # language is not changed to 'de' because the user already existed
+        # prior to the import
+        #
+        c['language'][email] = 'de'
+        before1, after1 = i.importer()
+        assert before1 == after0
+        assert before1 == after1
+        assert after1['preferences'][email]['locale'] == 'fr'
+    finally:
+        i.discourse.group_delete(i.group_moderators_name())
+        i.discourse.category_delete(i.name_get())
+        i.discourse.user_delete(email)
+
+
 def test_importer_category_info(test_options):
     i = importer.Importer(test_options)
     try:
